@@ -1,9 +1,10 @@
 package Git::ObjectStore;
-$Git::ObjectStore::VERSION = '0.003';
+$Git::ObjectStore::VERSION = '0.004';
 use strict;
 use warnings;
 
 use Git::Raw;
+use Git::Raw::Object;
 use Carp;
 use File::Spec::Functions qw(catfile);
 
@@ -315,19 +316,21 @@ sub recursive_read
     my $self = shift;
     my $path = shift;
     my $callback = shift;
-
+    my $no_content = shift;
+    
     croak('recursive_read() is called for a read-write ObjectStore object')
         if $self->{'writer'};
 
     if( $path eq '' )
     {
         foreach my $entry ($self->{'gittree'}->entries()) {
-            $self->_do_recursive_read($entry, $entry->name(), $callback);
+            $self->_do_recursive_read
+                ($entry, $entry->name(), $callback, $no_content);
         }
     } else {
         my $entry = $self->{'gittree'}->entry_bypath($path);
         if( defined($entry) ) {
-            $self->_do_recursive_read($entry, $path, $callback);
+            $self->_do_recursive_read($entry, $path, $callback, $no_content);
         }
         else
         {
@@ -344,17 +347,22 @@ sub _do_recursive_read
     my $entry = shift;  # Git::Raw::Tree::Entry object
     my $path = shift;
     my $callback = shift;
+    my $no_content = shift;
 
-    my $obj = $entry->object();
+    $entry->type();
 
-    if( $obj->is_tree() ) {
+    if( $entry->type() == Git::Raw::Object->TREE ) {
         # this is a subtree, we read it recursively
-        foreach my $child_entry ($obj->entries()) {
+        foreach my $child_entry ($entry->object()->entries()) {
             $self->_do_recursive_read
                 ($child_entry, $path . '/' . $child_entry->name(), $callback);
         }
     } else {
-        &{$callback}($path, $obj->content());
+        if( $no_content ) {
+            &{$callback}($path);
+        } else {
+            &{$callback}($path, $entry->object()->content());
+        }
     }
 
     return;
@@ -362,12 +370,15 @@ sub _do_recursive_read
 
 
 
+
+
 sub read_updates
 {
     my $self = shift;
     my $old_commit_id = shift;
     my $cb_updated = shift;
     my $cb_deleted = shift;
+    my $no_content = shift;
 
     my $old_commit = Git::Raw::Commit->lookup($self->{'repo'}, $old_commit_id);
     croak("Cannot lookup commit $old_commit_id") unless defined($old_commit);
@@ -393,8 +404,12 @@ sub read_updates
         if( $delta->status() eq 'deleted') {
             &{$cb_deleted}($path);
         } else {
-            my $entry = $new_tree->entry_bypath($path);
-            &{$cb_updated}($path, $entry->object()->content());
+            if( $no_content ) {
+                &{$cb_updated}($path);
+            } else {
+                my $entry = $new_tree->entry_bypath($path);
+                &{$cb_updated}($path, $entry->object()->content());
+            }
         }
     }
 
@@ -431,7 +446,7 @@ Git::ObjectStore - abstraction layer for Git::Raw and libgit2
 
 =head1 VERSION
 
-version 0.003
+version 0.004
 
 =head1 SYNOPSIS
 
@@ -601,20 +616,25 @@ This method combines C<create_commit()> and C<write_packfile>. The
 packfile is only written if there is a change in the content. The method
 returns true if any changes were detected.
 
-=head2 recursive_read($path, $callback)
+=head2 recursive_read($path, $callback, $no_content)
 
 This method is only supported in reader mode. It reads the directories
 recursively and calls the callback for every file it finds. The callback
 arguments are the file name and scalar content. If called with string as
-path, all files in the branch are traversed.
+path, all files in the branch are traversed. If the third argument is a
+true value, the method does not read the object contents, and the
+callback is only called with one argument.
 
-=head2 read_updates($old_commit_id, $callback_updated, $callback_deleted)
+=head2 read_updates($old_commit_id, $callback_updated,
+$callback_deleted, $no_content)
 
 This method is only supported in reader mode. It compares the current
 commit with the old commit, and executes the first callback for all
 added or updated files, and the second callback for all deleted
 files. The first callback gets the file name and scalar content as
-arguments, and the second callback gets only the file name.
+arguments, and the second callback gets only the file name. If the
+fourth argument is true, the update callback is called only with rhe
+file name.
 
 =head1 AUTHOR
 
